@@ -34,6 +34,7 @@ from pymongo import (ALL,
 from pymongo.collection import Collection
 from pymongo.database import Database
 from pymongo.errors import (CollectionInvalid,
+                            ConfigurationError,
                             InvalidName,
                             OperationFailure)
 from pymongo.son_manipulator import (AutoReference,
@@ -57,12 +58,16 @@ class TestDatabase(unittest.TestCase):
                           self.connection, u"my\u0000db")
         self.assertEqual("name", Database(self.connection, "name").name)
 
-    def test_cmp(self):
+    def test_equality(self):
         self.assertNotEqual(Database(self.connection, "test"),
                             Database(self.connection, "mike"))
         self.assertEqual(Database(self.connection, "test"),
                          Database(self.connection, "test"))
 
+        # Explicitly test inequality
+        self.assertFalse(Database(self.connection, "test") !=
+                         Database(self.connection, "test"))
+
     def test_repr(self):
         self.assertEqual(repr(Database(self.connection, "pymongo_test")),
                          "Database(%r, %s)" % (self.connection,
@@ -167,6 +172,8 @@ class TestDatabase(unittest.TestCase):
         self.assertRaises(ValueError, db.set_profiling_level, 5.5)
         self.assertRaises(ValueError, db.set_profiling_level, None)
         self.assertRaises(ValueError, db.set_profiling_level, -1)
+        self.assertRaises(TypeError, db.set_profiling_level, SLOW_ONLY, 5.5)
+        self.assertRaises(TypeError, db.set_profiling_level, SLOW_ONLY, '1')
 
         db.set_profiling_level(SLOW_ONLY)
         self.assertEqual(db.profiling_level(), SLOW_ONLY)
@@ -177,6 +184,15 @@ class TestDatabase(unittest.TestCase):
         db.set_profiling_level(OFF)
         self.assertEqual(db.profiling_level(), OFF)
 
+        db.set_profiling_level(SLOW_ONLY, 50)
+        self.assertEqual(50, db.command("profile", -1)['slowms'])
+
+        db.set_profiling_level(ALL, -1)
+        self.assertEqual(-1, db.command("profile", -1)['slowms'])
+
+        db.set_profiling_level(OFF, 100)  # back to default
+        self.assertEqual(100, db.command("profile", -1)['slowms'])
+
     def test_profiling_info(self):
         if is_mongos(self.connection):
             raise SkipTest('profile is not supported by mongos')
@@ -289,6 +305,13 @@ class TestDatabase(unittest.TestCase):
         db = self.connection.pymongo_test
         db.system.users.remove({})
         db.remove_user("mike")
+
+        self.assertRaises(TypeError, db.add_user, "user", None)
+        self.assertRaises(TypeError, db.add_user, "user", '')
+        self.assertRaises(TypeError, db.add_user, "user", 'password', None)
+        self.assertRaises(ConfigurationError, db.add_user,
+                          "user", 'password', 'True')
+
         db.add_user("mike", "password")
 
         self.assertRaises(TypeError, db.authenticate, 5, "password")
@@ -605,18 +628,20 @@ class TestDatabase(unittest.TestCase):
 
         self.assertRaises(OperationFailure, db.system_js.non_existant)
 
-        db.system_js.no_param = Code("return 5;")
-        self.assertEqual(5, db.system_js.no_param())
+        # XXX: Broken in V8, works in SpiderMonkey
+        if not version.at_least(db.connection, (2, 3, 0)):
+            db.system_js.no_param = Code("return 5;")
+            self.assertEqual(5, db.system_js.no_param())
 
     def test_system_js_list(self):
         db = self.connection.pymongo_test
         db.system.js.remove()
         self.assertEqual([], db.system_js.list())
 
-        db.system_js.foo = "blah"
+        db.system_js.foo = "function() { return 'blah'; }"
         self.assertEqual(["foo"], db.system_js.list())
 
-        db.system_js.bar = "baz"
+        db.system_js.bar = "function() { return 'baz'; }"
         self.assertEqual(set(["foo", "bar"]), set(db.system_js.list()))
 
         del db.system_js.foo
