@@ -34,15 +34,18 @@ from bson.tz_util import utc
 from pymongo.connection import Connection
 from pymongo.read_preferences import ReadPreference
 from pymongo.replica_set_connection import ReplicaSetConnection
-from pymongo.replica_set_connection import _partition_node
+from pymongo.mongo_replica_set_client import _partition_node
 from pymongo.database import Database
+from pymongo.pool import SocketInfo
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
                             ConnectionFailure,
                             InvalidName,
                             OperationFailure)
 from test import version
-from test.utils import delay, assertReadFrom, assertReadFromAll, read_from_which_host
+from test.utils import (
+    delay, assertReadFrom, assertReadFromAll, read_from_which_host,
+    assertRaisesExactly)
 
 
 host = os.environ.get("DB_IP", 'localhost')
@@ -98,7 +101,7 @@ class TestConnectionReplicaSetBase(unittest.TestCase):
 
 class TestConnection(TestConnectionReplicaSetBase):
     def test_connect(self):
-        self.assertRaises(ConnectionFailure, ReplicaSetConnection,
+        assertRaisesExactly(ConnectionFailure, ReplicaSetConnection,
                           "somedomainthatdoesntexist.org:27017",
                           replicaSet=self.name,
                           connectTimeoutMS=600)
@@ -551,7 +554,7 @@ class TestConnection(TestConnectionReplicaSetBase):
         self.assertNotEqual(0, cursor.cursor_id)
 
         connection_id = cursor._Cursor__connection_id
-        writer = c._ReplicaSetConnection__writer
+        writer = c._MongoReplicaSetClient__writer
         if read_pref == ReadPreference.PRIMARY:
             msg = "Expected cursor's connection_id to be %s, got %s" % (
                 writer, connection_id)
@@ -648,6 +651,46 @@ class TestConnection(TestConnectionReplicaSetBase):
             if old_signal_handler:
                 signal.signal(signal.SIGALRM, old_signal_handler)
 
+    def test_operation_failure_without_request(self):
+        # Ensure ReplicaSetConnection doesn't close socket after it gets an
+        # error response to getLastError. PYTHON-395.
+        c = self._get_connection(auto_start_request=False)
+        pool = c._MongoReplicaSetClient__members[self.primary].pool
+        self.assertEqual(1, len(pool.sockets))
+        old_sock_info = iter(pool.sockets).next()
+        c.pymongo_test.test.drop()
+        c.pymongo_test.test.insert({'_id': 'foo'}, safe=True)
+        self.assertRaises(
+            OperationFailure,
+            c.pymongo_test.test.insert, {'_id': 'foo'}, safe=True)
+
+        self.assertEqual(1, len(pool.sockets))
+        new_sock_info = iter(pool.sockets).next()
+
+        self.assertEqual(old_sock_info, new_sock_info)
+        c.close()
+
+    def test_operation_failure_with_request(self):
+        # Ensure ReplicaSetConnection doesn't close socket after it gets an
+        # error response to getLastError. PYTHON-395.
+        c = self._get_connection(auto_start_request=True)
+        c.pymongo_test.test.find_one()
+        pool = c._MongoReplicaSetClient__members[self.primary].pool
+
+        # Connection has reserved a socket for this thread
+        self.assertTrue(isinstance(pool._get_request_state(), SocketInfo))
+
+        old_sock_info = pool._get_request_state()
+        c.pymongo_test.test.drop()
+        c.pymongo_test.test.insert({'_id': 'foo'}, safe=True)
+        self.assertRaises(
+            OperationFailure,
+            c.pymongo_test.test.insert, {'_id': 'foo'}, safe=True)
+
+        # OperationFailure doesn't affect the request socket
+        self.assertEqual(old_sock_info, pool._get_request_state())
+        c.close()
+
     def test_auto_start_request(self):
         for bad_horrible_value in (None, 5, 'hi!'):
             self.assertRaises(
@@ -658,7 +701,7 @@ class TestConnection(TestConnectionReplicaSetBase):
         # auto_start_request should default to True
         conn = self._get_connection()
         pools = [mongo.pool for mongo in
-                 conn._ReplicaSetConnection__members.values()]
+                 conn._MongoReplicaSetClient__members.values()]
         self.assertTrue(conn.auto_start_request)
         self.assertTrue(conn.in_request())
 
@@ -693,22 +736,22 @@ class TestConnection(TestConnectionReplicaSetBase):
 
         secondaries = conn.secondaries
         for secondary in secondaries:
-            conn._ReplicaSetConnection__members[secondary].up = False
+            conn._MongoReplicaSetClient__members[secondary].up = False
 
-        conn._ReplicaSetConnection__members[conn.primary].up = False
+        conn._MongoReplicaSetClient__members[conn.primary].up = False
 
         # Wake up monitor thread
-        conn._ReplicaSetConnection__schedule_refresh()
+        conn._MongoReplicaSetClient__schedule_refresh()
 
         # Refresh interval is 30 seconds; scheduling a refresh tells the
         # monitor thread / greenlet to start a refresh now. We still need to
         # sleep a few seconds for it to complete.
         time.sleep(5)
         for secondary in secondaries:
-            self.assertTrue(conn._ReplicaSetConnection__members[secondary].up,
+            self.assertTrue(conn._MongoReplicaSetClient__members[secondary].up,
                 "ReplicaSetConnection didn't detect secondary is up")
 
-        self.assertTrue(conn._ReplicaSetConnection__members[conn.primary].up,
+        self.assertTrue(conn._MongoReplicaSetClient__members[conn.primary].up,
             "ReplicaSetConnection didn't detect primary is up")
 
         conn.close()
@@ -735,10 +778,18 @@ class TestConnection(TestConnectionReplicaSetBase):
         self.assertTrue(host in conn.secondaries)
         assertReadFrom(self, conn, host, ReadPreference.SECONDARY)
 
+        # Oddly, we expect PRIMARY_PREFERRED to keep reading from secondary,
+        # since the secondary is pinned and "matches" the preference.
+        assertReadFrom(self, conn, host, ReadPreference.PRIMARY_PREFERRED)
+
         # Repin
         primary = read_from_which_host(conn, ReadPreference.PRIMARY)
         self.assertEqual(conn.primary, primary)
         assertReadFrom(self, conn, primary, ReadPreference.NEAREST)
+        assertReadFrom(self, conn, primary, ReadPreference.PRIMARY_PREFERRED)
+
+        # Since the we're pinned to primary we still use it
+        assertReadFrom(self, conn, primary, ReadPreference.SECONDARY_PREFERRED)
 
         # Repin again
         host = read_from_which_host(conn, ReadPreference.SECONDARY)
